unit rpt1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, QuickRpt, Db, DBTables, Qrctrls, qrprntr, printers;

type
  PDeltaRec = ^aDeltaRec;
  aDeltaRec = record
    CustNo: integer;
    DetailCount: integer;
    TotalHeight: integer;
  end;

  TfrmRpt1 = class(TForm)
    QuickRep1: TQuickRep;
    Query1: TQuery;
    Query1OrderNo: TFloatField;
    Query1CustNo: TFloatField;
    Query1SaleDate: TDateTimeField;
    Query1ShipDate: TDateTimeField;
    Query1EmpNo: TIntegerField;
    Query1ShipToContact: TStringField;
    Query1ShipToAddr1: TStringField;
    Query1ShipToAddr2: TStringField;
    Query1ShipToCity: TStringField;
    Query1ShipToState: TStringField;
    Query1ShipToZip: TStringField;
    Query1ShipToCountry: TStringField;
    Query1ShipToPhone: TStringField;
    Query1ShipVIA: TStringField;
    Query1PO: TStringField;
    Query1Terms: TStringField;
    Query1PaymentMethod: TStringField;
    Query1ItemsTotal: TCurrencyField;
    Query1TaxRate: TFloatField;
    Query1Freight: TCurrencyField;
    Query1AmountPaid: TCurrencyField;
    DetailBand1: TQRBand;
    QRGroup1: TQRGroup;
    QRDBText1: TQRDBText;
    QRDBText2: TQRDBText;
    QRDBText3: TQRDBText;
    QRDBText4: TQRDBText;
    PageHeaderBand1: TQRBand;
    QRBand1: TQRBand;
    QRExpr1: TQRExpr;
    QRLabel1: TQRLabel;
    QRSysData1: TQRSysData;
    ColumnHeaderBand1: TQRBand;
    QRLabel2: TQRLabel;
    QRLabel3: TQRLabel;
    QRLabel4: TQRLabel;
    QRLabel5: TQRLabel;
    procedure DetailBand1AfterPrint(Sender: TQRCustomBand;
      BandPrinted: Boolean);
    procedure QuickRep1BeforePrint(Sender: TCustomQuickRep;
      var PrintReport: Boolean);
    procedure QRGroup1AfterPrint(Sender: TQRCustomBand;
      BandPrinted: Boolean);
    procedure QRGroup1BeforePrint(Sender: TQRCustomBand;
      var PrintBand: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    MyCounter: integer;               // A counter so that we can link GroupHeightList to the current band
    FPageFooterSize: extended;        // Size of the footer band
    procedure MyResetPageFooterSize;  // Code to get FPageFooterSize
    procedure ClearGroupHeightList;   // Clean up code
  public
    { Public declarations }
    CheckDelta: boolean;    // Are we doing the 1st pass
    GroupHeightList: TList; // List of stretched heights of the detail bands
  end;

var
  frmRpt1: TfrmRpt1;

implementation

{$R *.DFM}

// This code is based on the ResetPageFooterSize procedure in
// quickrep.pas.  We need this here because the code in quickrep
// is private and can not be accessed directly
procedure TfrmRpt1.MyResetPageFooterSize;
var
  I : integer;
  aBand : TQRCustomBand;
begin
  FPageFooterSize := 0;
  with QuickRep1 do
    for I := 0 to ControlCount - 1 do
    begin
      if Controls[I] is TQRCustomBand then
        with TQRCustomBand(Controls[I]) do
        begin
          if (BandType = rbPageFooter) and Enabled then
          begin
            FPageFooterSize := FPageFooterSize + Size.Length;
            aBand := TQRCustomBand(QuickRep1.Controls[I]);
            while aBand.HasChild do
            begin
              aBand := aBand.ChildBand;
              FPageFooterSize := FPageFooterSize + aBand.Size.Length;
            end;
          end;
        end
    end
end;

procedure TfrmRpt1.QuickRep1BeforePrint(Sender: TCustomQuickRep;
  var PrintReport: Boolean);
begin
  // Clear the list if running the first pass
  if CheckDelta then
    ClearGroupHeightList;

  // To make it easier to measure the length of a band, we put each detailband
  // on it's own page.  This should be doen for the first pass.  If the final report
  // requires this, just leave it set to True.
  QRGroup1.ForceNewPage := CheckDelta;
  DetailBand1.ForceNewPage := CheckDelta;

  // Get the footer size
  MyResetPageFooterSize;

  // reset the counter used to go through the GroupHeightList list
  MyCounter := 0;
end;

procedure TfrmRpt1.QRGroup1AfterPrint(Sender: TQRCustomBand;
  BandPrinted: Boolean);
var
  ARecord: PDeltaRec;
begin
  if CheckDelta then
  begin
    // create a new record
    new(ARecord);
    // Start with the height of the group band.  If you
    ARecord^.TotalHeight := round(QRGroup1.Size.Length + QRGroup1.Expanded);

    // These two fields are not used in this report.  You could use fields
    // DetailCount to display the number for detail lines in the group header
    ARecord^.CustNo := Query1CustNo.AsInteger;
    ARecord^.DetailCount := 0;

    // Add this record to the list
    GroupHeightList.Add(ARecord);
  end;
end;

procedure TfrmRpt1.QRGroup1BeforePrint(Sender: TQRCustomBand;
  var PrintBand: Boolean);
var
  ARecord: PDeltaRec;
  AvailableSpace: integer;
begin
  // final or only pass
  if not CheckDelta then
  begin
    // Make sure that we did a first pass
    if GroupHeightList.Count > 0 then
    begin
      try
        // Get the record for the current group
        ARecord := GroupHeightList.Items[MyCounter];

        inc(MyCounter);

        // Check how much space is left on the page
        with QuickRep1 do
          if Page.Orientation = poPortrait then
            AvailableSpace := round(QRPrinter.PaperLength - Page.BottomMargin - FPageFooterSize - CurrentY)
          else
            AvailableSpace := round(QRPrinter.PaperWidth - Page.BottomMargin - FPageFooterSize - CurrentY);

        // We use 10% of the band height as "fudge" factor, otherwise the page footer
        // band will overwrite the bottom line of the band.  You may need to
        // adjust this value.
        if AvailableSpace < ARecord^.TotalHeight + (ARecord^.TotalHeight / 10)then
          QuickRep1.NewColumn;
      except
        // You shouldn't get an exception, but if you do...
        raise;
      end;
    end;
  end;

end;

procedure TfrmRpt1.DetailBand1AfterPrint(Sender: TQRCustomBand;
  BandPrinted: Boolean);
var
  ARecord: PDeltaRec;
begin
  // On the first pass, we get the height of the band and add it to
  // the GroupHeightList list
  if CheckDelta then
  begin
    // The last record in the list is the current one
    ARecord := GroupHeightList.Items[GroupHeightList.Count-1];

    // Add the height of this band to record.  Notice that we use Sender
    // instead of the specific band.  This lets us use the same event for
    // of the bands in this group (except the group header)
    ARecord^.TotalHeight := ARecord^.TotalHeight + round(Sender.Size.Length + Sender.Expanded);
    // optional information - not used by this report
    ARecord^.DetailCount := ARecord^.DetailCount + 1;
  end;
end;


procedure TfrmRpt1.FormCreate(Sender: TObject);
begin
  // Create the list to store the band heights
  GroupHeightList := TList.Create;
  // Default to not checking the height.
  CheckDelta := false;
end;

procedure TfrmRpt1.ClearGroupHeightList;
var
  ARecord: PDeltaRec;
  i: integer;
begin
  // Free the memory used by the records in a list.
  for i := 0 to GroupHeightList.Count-1 do begin
    ARecord := GroupHeightList.Items[i];
    Dispose(ARecord);
  end;
  GroupHeightList.Clear;
end;

procedure TfrmRpt1.FormDestroy(Sender: TObject);
begin
  // Free the list
  ClearGroupHeightList;
  GroupHeightList.Free;
end;

end.
